(******************************************************************************
 * AcComboBox                                                                 *
 ******************************************************************************)
unit ACComboBox;
{$i AcDefinizioni.pas }
(******************************************************************************
 * Interface                                                                  *
 ******************************************************************************)
interface

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,CommCtrl
{$IFDEF DELPHI6}
  ,ImgList
{$ENDIF}
  ;

(******************************************************************************
 * Type                                                                       *
 ******************************************************************************)
type
  TACComboBox = class(TCustomComboBox)
  private
    { Private declarations }
  protected
    { Protected declarations }
    Bmp              : TBitMap;
    FListImage       : TImageList;
    FImageChangeLink : TChangeLink;
    FImageIndex      : TStringList;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetImages(Value: TImageList);
    procedure SetImageIndex(Value: TStringList);
    procedure SetImageList(Value: HImageList; Flags: Integer);
    procedure ACComboDrawItem(Control: TWinControl; Index: Integer;Rect: TRect;
                              State: TOwnerDrawState);
    procedure ACComboMeasureItem(Control: TWinControl;Index: Integer;
                              var Height: Integer);

  published
    { Published declarations }
//    property Style; {Must be published before Items}
    Property ImageList : TImageList Read FListImage Write SetImages;
    Property ImageIndex : TStringList Read FImageIndex Write SetImageIndex;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
    property ItemHeight;
    property Items;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDrag;

  end;


(******************************************************************************
 * implementation                                                             *
 ******************************************************************************)
implementation


(******************************************************************************
 * Constructor                                                                *
 ******************************************************************************)
constructor TACComboBox.Create(AOwner: TComponent);
Begin
  Inherited;
  FImageChangeLink := TChangeLink.Create;
  FListImage       := TImageList.CreateSize(32, 32);
  Bmp              := TBitMap.Create;
  Style            := csOwnerDrawVariable;
  OnDrawItem       := ACComboDrawItem;
  FImageIndex      := TStringList.Create;
End;

(******************************************************************************
 * Destructor                                                                 *
 ******************************************************************************)
destructor TACComboBox.Destroy;
begin
  ImageList := Nil;
  Bmp.Free;
  FListImage.Free;
  FImageChangeLink.Free;
  FImageIndex.Free;
  Inherited;
end;

(******************************************************************************
 * SetImages                                                                  *
 ******************************************************************************)
procedure TACComboBox.SetImages(Value: TImageList);
begin
  if ImageList <> nil then
    FListImage.UnRegisterChanges(FImageChangeLink);
  FListImage := Value;
  if FListImage <> nil then
  begin
    FListImage.RegisterChanges(FImageChangeLink);
    SetImageList(FListImage.Handle, TVSIL_NORMAL)
  end
  else SetImageList(0, TVSIL_NORMAL);
end;

(******************************************************************************
 * SetImageList                                                               *
 ******************************************************************************)
procedure TACComboBox.SetImageList(Value: HImageList; Flags: Integer);
begin
  if HandleAllocated then ListView_SetImageList(Handle, Value, Flags);
end;

(******************************************************************************
 * AcComboDrawItem                                                            *
 ******************************************************************************)
procedure TACComboBox.ACComboDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var Indice : Integer;
    Larghezza : Integer;
begin
  Canvas.FillRect(Rect);
  Bmp.Canvas.Brush.Color := Color;
  Bmp.Canvas.FillRect(ClientRect);
  if (ImageIndex.Count-1>=Index) then Indice := StrToInt(ImageIndex[Index])
  else Indice := -1;
  ImageList.GetBitMap(Indice,Bmp);
//  Larghezza := 0;
  if (Indice>=0) then
  begin
    Larghezza := Bmp.Width+1;
    Canvas.Draw(Rect.Left,Rect.Top,Bmp);
  end
  Else Larghezza := Bmp.Width+1;
  Canvas.TextOut(Rect.Left+Larghezza,Rect.Top,Items[index]);

end;

(******************************************************************************
 * AcComboMeasureItem                                                         *
 ******************************************************************************)
procedure TACComboBox.ACComboMeasureItem(Control: TWinControl;Index: Integer;
          var Height: Integer);
var Indice : Integer;
begin
  if (ImageIndex.Count-1>=Index) then Indice := StrToInt(ImageIndex[Index])
  else Indice := -1;
  if (Indice>=0) then
  begin
    ImageList.GetBitMap(Indice,Bmp);
    Height := Bmp.Height;
  end;
end;

(******************************************************************************
 * SetImageIndex                                                              *
 ******************************************************************************)
procedure TACComboBox.SetImageIndex(Value: TStringList);
Begin
  FImageIndex.Assign(Value);
End;
end.
