(******************************************************************************
 * AcDiskInfo                                                                 *
 ******************************************************************************)
unit ACDiskInfo;
{$i AcDefinizioni.pas }
(******************************************************************************
 * Interface                                                                  *
 ******************************************************************************)
interface

(******************************************************************************
 *                                                                            *
 ******************************************************************************)
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

(******************************************************************************
 *                                                                            *
 ******************************************************************************)
type
  TACDiskInfoType = (AC_Drive_Undetermined,
                     AC_Drive_NotExist,
                     AC_Drive_Removable,
                     AC_Drive_Fixed,
                     AC_Drive_Remote,
                     AC_Drive_CdRom,
                     AC_Drive_RamDisk);
  TACDiskInfo = class(tcomponent)
  private
    { Private declarations }
    Lettera_Corrente : String;
    f_lpSectorsPerCluster     : Integer;
    f_lpBytesPerSector        : Integer;
    f_lpNumberOfFreeClusters  : Integer;
    f_lpTotalNumberOfClusters : Integer;
    f_DiskType                : TACDiskInfoType;
    f_lpVolumeNameBuffer,
    f_lpFileSystemNameBuffer   : string;
{$IFDEF DELPHI2}
    f_lpMaximumComponentLength,
    f_lpFileSystemFlags        : Integer;
    f_lpVolumeSerialNumber     : Integer;
{$ENDIF}
{$IFDEF DELPHI6}
    f_lpMaximumComponentLength,
    f_lpFileSystemFlags        : LongWord;
    f_lpVolumeSerialNumber     : LongWord;
{$ENDIF}

  protected
    { Protected declarations }
    function Get_Drives : String;
    Procedure Void_Write_Str(txt:String);
    function Get_Value_Str(Ndx : Integer) : String;
    Procedure Set_Value_Str(Ndx : Integer;Txt:String);
//    function Get_Value_Int(Ndx : Integer) : Integer;
    Procedure Set_Valori_Disco;
    Procedure Set_Void_Type(X : TACDiskInfoType);
    Procedure Set_VoidValue_Int(X:Integer);
{$IFDEF DELPHI6}
    Procedure Set_VoidValue_LongInt(X:LongWord);
{$ENDIF}



  public
    { Public declarations }
    constructor Create(AOwner: TComponent); Override;
  published
    { Published declarations }
    Property Drives                : String Read Get_Drives Write Void_Write_Str;
    Property Drive                 : String Index 0 Read Get_Value_Str Write Set_Value_Str;
    Property DiskType              : TACDiskInfoType Read f_DiskType Write Set_Void_Type;
    Property SectorsPerCluster     : Integer Read f_lpSectorsPerCluster Write Set_VoidValue_Int;
    Property BytesPerSector        : Integer Read f_lpBytesPerSector Write Set_VoidValue_Int;
    Property NumberFreeClusters    : Integer Read f_lpNumberOfFreeClusters Write Set_VoidValue_Int;
    Property TotalClusters         : Integer Read f_lpTotalNumberOfClusters Write Set_VoidValue_Int;
    Property VolumeName            : String Read f_lpFileSystemNameBuffer Write Void_Write_Str;
{$IFDEF DELPHI2}
    Property VolumeSerialNumber    : Integer Read f_lpVolumeSerialNumber Write Set_VoidValue_Int;
{$ENDIF}
{$IFDEF DELPHI6}
//    Property VolumeSerialNumber    : LongWord Read f_lpVolumeSerialNumber Write Set_VoidValue_LongInt;
{$ENDIF}
  end;


(******************************************************************************
 * Implementation                                                             *
 ******************************************************************************)
implementation

(******************************************************************************
 * Get_Drives                                                                 *
 ******************************************************************************)
function TACDiskInfo.Get_Drives : String;
var c1,c2 : Integer;
    txt   : String;
begin
 txt := '';
 c1 := GetLogicalDrives;
 for c2 := 1 to 32 do
 begin
   if (c1 and 1<>0) then txt := txt + chr(64+c2);
   c1 := c1 div 2;
 end;
 Get_Drives := Txt;

End;
(******************************************************************************
 *                                                                            *
 ******************************************************************************)
Procedure TACDiskInfo.Void_Write_Str(txt:String);
Begin
End;

(******************************************************************************
 *                                                                            *
 ******************************************************************************)
function TACDiskInfo.Get_Value_Str(Ndx : Integer) : String;
Begin
  Get_Value_Str := '';
  Case Ndx of
//********** Unita' selezionata **********
    0:
      Begin
        Get_Value_Str := Lettera_Corrente;
      End;
  End;
End;

(******************************************************************************
 *                                                                            *
 ******************************************************************************)
Procedure TACDiskInfo.Set_Value_Str(Ndx : Integer;Txt:String);
Begin
  Case Ndx of
//********** Unita' selezionata **********
    0:
      Begin
        Lettera_Corrente := Txt;
        Set_Valori_Disco;
      End;
  End;
End;


(******************************************************************************
 * Constructor                                                                *
 ******************************************************************************)
constructor TACDiskInfo.Create(AOwner: TComponent);
Begin
  Inherited;
  if (Drives<>'') Then
    Lettera_Corrente := Drives[1]
  Else Lettera_Corrente := '';
End;


(******************************************************************************
 * Set_Valori_Disco                                                           *
 ******************************************************************************)
Procedure TACDiskInfo.Set_Valori_Disco;
Var
{$IFDEF DELPHI2}
    v1,v2,v3,v4 : Integer;
{$ENDIF}
{$IFDEF DELPHI6}
    v1,v2,v3,v4 : Longword;
{$ENDIF}
    Percorso    : Array[0..5] of char;
    txt1,
    txt2         : array[0..64] of char;
    app1         : DWord;
    fl           : Boolean;
Begin
  if (Trim(Drive)<>'') Then
  Begin
    StrPCopy(Percorso,Drive+':\'+#0);
//********** Tipo unit *********
    v1 := GetDriveType(Percorso);
    f_DiskType := AC_Drive_Undetermined;
    Case v1 of
      1:                       f_DiskType := AC_Drive_NotExist;
      Windows.DRIVE_REMOVABLE: f_DiskType := AC_Drive_Removable;
      Windows.DRIVE_FIXED:     f_DiskType := AC_Drive_Fixed;
      Windows.DRIVE_REMOTE:    f_DiskType := AC_Drive_Remote;
      Windows.DRIVE_CDROM:     f_DiskType := AC_Drive_CdRom;
      Windows.DRIVE_RAMDISK:   f_DiskType := AC_Drive_RamDisk;
    End;
//********** GetDiskFreeSpace **********

    GetDiskFreeSpace(Percorso,v1,v2,v3,v4);
    f_lpSectorsPerCluster     := v1;
    f_lpBytesPerSector        := v2;
    f_lpNumberOfFreeClusters  := v3;
    f_lpTotalNumberOfClusters := v4;
//********** Informazioni sul volume **********

    fl:=GetVolumeInformation(Percorso,
                             txt1,	                // address of name of the volume
                             64,	                // length of lpVolumeNameBuffer
                             @app1,	                // address of volume serial number
                             f_lpMaximumComponentLength,// address of system's maximum filename length
                             f_lpFileSystemFlags,	// address of file system flags
                             txt2,	                // address of name of file system
                             64); 	                // length of lpFileSystemNameBuffer
    if (fl) then
    Begin
      f_lpVolumeNameBuffer       := txt1;
      f_lpFileSystemNameBuffer   := txt2;
      f_lpVolumeSerialNumber     := App1;
    End
    Else
    Begin
      f_lpVolumeNameBuffer       := '';
      f_lpFileSystemNameBuffer   := '';
      f_lpVolumeSerialNumber     := 0;
      f_lpMaximumComponentLength := 0;
      f_lpFileSystemFlags        := 0;

    End;
  End;

End;

(******************************************************************************
 * Set_Void_Type                                                              *
 ******************************************************************************)
Procedure TACDiskInfo.Set_Void_Type(X : TACDiskInfoType);
Begin
End;

(******************************************************************************
 * Set_VoidValue_Int                                                          *
 ******************************************************************************)
Procedure TACDiskInfo.Set_VoidValue_Int(X:Integer);
Begin
End;

(******************************************************************************
 * Set_VoidValue_LongInt                                                      *
 ******************************************************************************)
{$IFDEF DELPHI6}
Procedure TACDiskInfo.Set_VoidValue_LongInt(X:LongWord);
Begin
End;
{$ENDIF}


End.

