(******************************************************************************
 * AcQrCustCtrl                                                               *
 ******************************************************************************)
unit AcQrCustCtrl;

(******************************************************************************
 * Interface                                                                  *
 ******************************************************************************)
interface

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
Uses Controls, QuickRep, Classes, Graphics, Windows, AcProcDeclare;

(******************************************************************************
 * Type                                                                       *
 ******************************************************************************)
Type
  TACQRCustomGraph = class(TQRCustomControl)
  private
     FAlignToBand  : Boolean;
     FOnPrintEvent : TQRLabelOnPrintEvent;
     FReport       : TQuickReport;
     DatiLinee     : TStringList;
     FPen          : TPen;
     FBrush        : TBrush;

  protected

     property OnPrint : TQRLabelOnPrintEvent read FOnPrintEvent Write FOnPrintEvent;
     Procedure Ottiene_Dati(Txt:String;num:Integer;Var dati : array of Integer);

  public
     constructor Create(AOwner : TComponent); override;
     procedure Paint; override;
     procedure Print(X,Y : Integer); Override;
     destructor Destroy; Override;
     Procedure Set_Linee(txt:TStringList);
     Function Moltiplica_per_Uno(Numero:Longint):Longint;
     Procedure Disegna(Video:Boolean;xx,yy:Integer;fnzX,fnzY:AcFunMolt;Cnv:TCanvas);
     procedure SetBrush(Value: TBrush);
     procedure SetPen(Value: TPen);


//     procedure Stretch(var size : Integer); virtual;
//     property AutoSize Stored true;
//     property ParentReport : TQuickReport read FReport write FReport;
  protected
     property Height;
     property Left;
     property Top;
     property Visible;
     property Width;
     Property Linee : TStringList Read DatiLinee Write Set_Linee;
     property Brush: TBrush read FBrush write SetBrush;
     property Pen: TPen read FPen write SetPen;
     procedure StyleChanged(Sender: TObject);
//
  published
//   property AlignToBand : Boolean read FAlignToBand write FAlignToBand;
  end;

(******************************************************************************
 * Implementation                                                             *
 ******************************************************************************)
implementation

(******************************************************************************
 * Uses                                                                       *
 ******************************************************************************)
Uses SysUtils;


  procedure PrintBitmap(aCanvas : TCanvas; Dest : TRect; Bitmap : Graphics.TBitmap);
  var
    Info : PBitmapInfo;
    InfoSize : integer;
    Image : Pointer;
{$ifndef ver90}
    ImageSize : DWord;
{$else}
  {$ifndef ver93}
     ImageSize : DWord;
  {$else}
     ImageSize : Longint;
  {$endif}
{$endif}
  begin
    Info := Nil;
    Image := Nil;
    with Bitmap do
    begin
      GetDIBSizes(Handle, InfoSize, ImageSize);
      try
        Info := AllocMem(InfoSize);
        try
          Image := AllocMem(ImageSize);
          GetDIB(Handle, Palette, Info^, Image^);
          if not Monochrome then
            SetStretchBltMode(ACanvas.Handle, STRETCH_DELETESCANS);
          with Info^.bmiHeader do
            StretchDIBits(aCanvas.Handle, Dest.Left, Dest.Top,
              Dest.RIght - Dest.Left, Dest.Bottom - Dest.Top,
              0, 0, biWidth, biHeight, Image, Info^, DIB_RGB_COLORS, SRCCOPY);
        finally
          FreeMem(Image, ImageSize);
        end;
      finally
        FreeMem(Info, InfoSize);
      end;
    end;
  end;

(******************************************************************************
 * Disegna                                                                    *
 ******************************************************************************)
Procedure TACQRCustomGraph.Disegna(Video:Boolean;xx,yy:Integer;fnzX,fnzY:AcFunMolt;Cnv:TCanvas);
Type
  PPoints = ^TPoints;
  TPoints = array[0..0] of TPoint;
Var dati : array [1..4] of Integer;
    c1   : Integer;
    Punti : PPoints;
    Conta : Integer;
    
Begin
  if (not Visible) Then Exit;
  Conta := 0;
  GetMem(Punti,Linee.Count*Sizeof(TPoint));
  Cnv.Pen := Pen;
  Cnv.Brush := Brush;

  For c1 := 0 to Linee.Count-1 Do
  Begin
    if (Linee[c1]<>'') then
    Begin
      if (Linee[c1][1]='L') then
      Begin
        Ottiene_Dati(Linee[c1],2,Dati);
        Punti[Conta].X := fnzX(xx+dati[1]);
        Punti[Conta].Y := fnzY(yy+dati[2]);
        Conta := conta + 1;
      End;
    End;

  End;
  Cnv.PolyGon(Slice(Punti^, Conta));
End;

(******************************************************************************
 * Paint                                                                      *
 ******************************************************************************)
procedure TACQRCustomGraph.Paint;
Begin
  Disegna(True,0,0,Moltiplica_per_Uno,Moltiplica_per_Uno,Canvas);
End;

(******************************************************************************
 * Constructor                                                                *
 ******************************************************************************)
constructor TACQRCustomGraph.Create(AOwner : TComponent);
Begin
 Inherited;
 FPen            := TPen.Create;
 FBrush          := TBrush.Create;
 Brush.Color     := clTeal;
 ControlStyle    := ControlStyle-[csOpaque];
 Caption         := ' ';
 DatiLinee       := TStringList.Create;
 Height          := 100;
 Width           := 100;
 FPen.OnChange   := StyleChanged;
 FBrush.OnChange := StyleChanged;

End;

(******************************************************************************
 * Print                                                                      *
 ******************************************************************************)
procedure TACQRCustomGraph.Print(X,Y : Integer);
Begin
// Inherited;
 Disegna(False,Left+X,Top+Y,ParentReport.XPos,ParentReport.YPos,QRPrinter.Canvas);
End;


(******************************************************************************
 * Destroy                                                                    *
 ******************************************************************************)
destructor TACQRCustomGraph.Destroy;
Begin
 DatiLinee.Free;
 FPen.Free;
 FBrush.Free;

 Inherited;
End;

(******************************************************************************
 * Ottiene_Dati                                                               *
 ******************************************************************************)
Procedure TACQRCustomGraph.Ottiene_Dati(Txt:String;num:Integer;Var dati : array of Integer);
Var TxtDati    : array[1..10] of string;
    c1,p1      : Integer;
Begin
  Delete(txt,1,1);
  txt := txt + ',';
  For c1 := 1 to 10 do TxtDati[c1] := '';
  For c1 := 1 to Num do
  Begin
    if (Txt<>'') then
    Begin
      p1          := Pos(',',Txt);
      TxtDati[c1] := Copy(Txt,1,p1-1);
      Delete(Txt,1,p1);
    End;
  End;
  For c1 := 0 to num-1 Do // Gli array passati ome quello in parametro iniziano da 0
    Dati[c1] := StrToInt(TxtDati[c1+1]); // anche se era stato dihiarato a partire da 1
End;

(******************************************************************************
 * Moltiplica_Per_Uno                                                         *
 ******************************************************************************)
Function TACQRCustomGraph.Moltiplica_per_Uno(Numero:Longint):Longint;
Begin
  Moltiplica_per_Uno := Numero;
End;


(******************************************************************************
 * Set_Lines                                                                  *
 ******************************************************************************)
Procedure TACQRCustomGraph.Set_Linee(txt:TStringList);
Begin
  DatiLinee.Assign(txt);
End;

(******************************************************************************
 * SetPen                                                                     *
 ******************************************************************************)
procedure TACQRCustomGraph.SetPen(Value: TPen);
begin
  FPen.Assign(Value);
end;

(******************************************************************************
 * SetBrush                                                                   *
 ******************************************************************************)
procedure TACQRCustomGraph.SetBrush(Value: TBrush);
begin
  FBrush.Assign(Value);
end;

(******************************************************************************
 * StyleChanged                                                               *
 ******************************************************************************)
procedure TACQRCustomGraph.StyleChanged(Sender: TObject);
begin
  Invalidate;
end;


end.
